#!/usr/bin/env python
#-*- coding:utf-8 –*-
#-----------------------------------------------------------------------------
# $Header: $
#-----------------------------------------------------------------------------
# Summary:           Implementation of DC Measure
#
# Authors:           Jensen.Cai
#
# Started:           11/25/2014
#
# Version:           V1.7R1
#
# Copyright 2013-2016 Siglent Corporation. All Rights Reserved.
#
#-----------------------------------------------------------------------------

import sys, time
sys.path.append(sys.path[0]+'\\Prod Lib')
from reminder import popup
from test_info import testInfo_show
from win32com_excel import *
from device_remote import *
from param import *
from file import *

#-----------------------------------------------------------------------------
class DCMeasure(DCParam):
    def __init__(self):
        super(DCMeasure, self).__init__()
        self.DC = DCRemote()
        self.DMM = DMMRemote()
        self.fail_info = []
        self.row = 9
    
    def DCTest_main(self):        
        self.DMM.set_default()
        self.DMM.set_mode('DC')

        #生成报告文件并将设备信息写入文件头
        report = TestReport('DCMeasure')
        xls_name = report.create()
        self.xls = easyExcel(xls_name)
        report.head_write(self.xls)

        print '\nStarting DC Measure Process...\n'

        for ch in self.DC_ch_list:
            self.DC.set_output_load(ch, 'HZ')
            self.DC.set_wvtp(ch, 'DC')
            self.DC.set_ofst(ch, 0)
            popup(u'注意! Notice!', \
                  u'请将万用表连接至%s 并断开另一通道\n\nPlease connect the DMM to %s and disconnect another channel' % (ch,  ch))
            self.DC.set_output_on(ch)
            if ch == 'C1':
                self.__DC_Meas(ch, self.DC_ch1_dclist)
            elif ch == 'C2':
                self.__DC_Meas(ch, self.DC_ch2_dclist)
            self.DC.set_output_off(ch)
    
    def __DC_Meas(self, ch_meas, DC_list):
        #直流检测
        for DC_set in DC_list:
            DC_min = DC_set - abs(DC_set) * 0.01 - self.DC_err
            DC_max = DC_set + abs(DC_set) * 0.01 + self.DC_err
            self.DC.set_ofst(ch_meas, DC_set)
            DC_read = self.DMM.get_value('DC')
            if DC_read > DC_min and DC_read < DC_max:
                result = 'Pass'
            else:
                result = 'Fail'
                self.fail_info.append(u'%s 直流档位%sV检测失败' % (ch_meas, DC_set))
                self.fail_info.append('DC Measure:%s %sV Fail' % (ch_meas, DC_set))
            print '%s: SetDC=%sV, MeasuredDC=%.6fV, %s' % (ch_meas, DC_set, DC_read, result)
            self.xls.setCell('Sheet1', self.row, 1, '%s' % ch_meas)
            self.xls.setCell('Sheet1', self.row, 2, '%s' % DC_set)
            self.xls.setCell('Sheet1', self.row, 3, '%s' % DC_read)
            self.xls.setCell('Sheet1', self.row, 4, '%s ~ %s' % (DC_min, DC_max))
            self.xls.setCell('Sheet1', self.row, 5, '%s' % result)
            self.row += 1
            
    def saveAndClose(self):
        #保存报告文件并关闭
        self.xls.save()
        time.sleep(0.2)
        self.xls.close()
        time.sleep(0.2)

        self.DC.close()
        self.DMM.close()
        time.sleep(0.2)
        print 'The DC Measure Process is done'

    def show_result(self):
        if self.fail_info == []:
            testInfo_show([u'直流精度测量通过，请进行下一步操作', 'DC Measure: Pass'])
        else:
            self.fail_info.insert(0, u'直流精度测量失败\n')
            self.fail_info.insert(1, 'DC Measure: Fail\n')
            testInfo_show(self.fail_info)
            
if __name__ == '__main__':
    proc = DCMeasure()
    proc.DCTest_main()
    proc.saveAndClose()
    proc.show_result()
