#!/usr/bin/env python
#-*- coding:utf-8 –*-
#-----------------------------------------------------------------------------
# $Header: $
#-----------------------------------------------------------------------------
# Summary:           Implementation of SDG1000X Square Wave Duty Calibration
#
# Authors:           Gary.Huang
#
# Started:           2/25/2015
#
# Version:           V1.7R1
#
# Copyright 2013-2016 Siglent Corporation. All Rights Reserved.
#
#-----------------------------------------------------------------------------

import sys, time, math
sys.path.append(sys.path[0]+'\\Prod Lib')
from reminder import popup
from test_info import testInfo_show
from win32com_excel import *
from device_remote import *
from param import *
from file import *

#-----------------------------------------------------------------------------
class DutyCalib_x(SqrDutyParam):
    def __init__(self):
        super(DutyCalib_x, self).__init__()
        self.SqrDuty = DutyRemote()
        self.SDS = SDSRemote()
        self.SDS.set_default()
        self.fail_info = []

    def DutyCal_main(self):
        self.SDS.set_dutyMeas(self.adj_sch)

        #生成报告文件并将设备信息写入文件头
        report = TestReport('DutyCalib')
        xls_name = report.create()
        self.xls = easyExcel(xls_name)
        report.head_write(self.xls)

        print '\nStarting Square Wave Duty Calibrate Process...\n'

        self.row = 8
        
        self.__SqrWaveDuty_calib()
    
        self.row = 90
        self.__SqrWaveDuty_test()
        
        self.SDS.change_dutyStat()
        for ch in self.sqrDuty_ch_list:
            self.SqrDuty.set_output_load(ch, 'HZ')

    def __SqrWaveDuty_calib(self):
        #开始占空比校正
        for ch in self.sqrDuty_ch_list:
            popup(u'注意! Notice!',u'请将示波器连接至%s 并断开另一通道\n\nPlease connect the DSO to %s and disconnect another channel' % (ch, ch))
            self.SqrDuty.set_output_on(ch)
    
            self.SqrDuty.set_output_load(ch, '50')
            self.SqrDuty.set_bswv(ch, 'SQUARE', 30000000, 4, 0, 0)
            for i in range(2):
                print '%s%% Duty Calibrating:\n' % self.dutyCal_duty[i]
                self.set_duty[i] = self.__duty_calib(ch, self.dutyCal_duty[i])
                self.__calData_check(self.dutyCal_duty[i], self.set_duty[i])
                self.row += 20
            self.save_calCurve(ch)
            
            self.SqrDuty.set_output_off(ch)
        
        self.SqrDuty.save_adjData()
        time.sleep(self.save_time)

    def __calData_check(self, duty_cal, duty_save):
        if duty_save>duty_cal+10 or duty_save<duty_cal-10:
            raw_input('duty cycle calibration error:out of range!')
        else:
            pass
    
    def save_calCurve(self, cal_ch):
        def_cmpLvl = [-0.117, 0.117]
        set_cmpLvl = [-0.117, 0.117]
        for j in range(0, 2):
            def_cmpLvl[j] = self.__cmpLvl_calc(self.dutyCal_duty[j])
            set_cmpLvl[j] = self.__cmpLvl_calc(self.set_duty[j])
        slope = (set_cmpLvl[1] - set_cmpLvl[0]) / (def_cmpLvl[1] - def_cmpLvl[0])
        delta = (set_cmpLvl[1] + set_cmpLvl[0]) / 2
        self.SqrDuty.set_calData(cal_ch, round(delta, 6), round(slope, 6))
        self.SqrDuty.save_adjData()
        time.sleep(self.save_time)

    def __cmpLvl_calc(self, duty_calc):
        amplitude=0.757/2
        cmpLvl_calc = amplitude*math.sin((duty_calc-50)*math.pi/100)
        return cmpLvl_calc

    def __duty_calib(self, ch_adj, dest_duty):
        #软件调整内部设置值，直到测量得到的占空比在误差范围之内即可
        min_duty = dest_duty - self.dutyCal_err
        max_duty = dest_duty + self.dutyCal_err
        test_duty = dest_duty
        mean_duty = self.__duty_meas(ch_adj, test_duty)
        cnt = 1
        print 'CalibrateTimes:%s, DestDuty=%f, MeanDuty=%f' % (cnt, dest_duty, mean_duty)
        self.xls.setCell('Sheet1', self.row + cnt, 1, '%s' % ch_adj)
        self.xls.setCell('Sheet1', self.row + cnt, 2, '%s' % dest_duty)
        self.xls.setCell('Sheet1', self.row + cnt, 3, '%s' % cnt)
        self.xls.setCell('Sheet1', self.row + cnt, 4, '%s' % test_duty)
        self.xls.setCell('Sheet1', self.row + cnt, 5, '%s' % mean_duty)
        self.xls.setCell('Sheet1', self.row + cnt, 6, '%s ~ %s' % (min_duty, max_duty))
        while mean_duty < min_duty or mean_duty > max_duty:
            duty_step = (mean_duty - dest_duty) / 2
            test_duty = test_duty - duty_step
            mean_duty = self.__duty_meas(ch_adj, test_duty)
            cnt += 1
            print 'CalibrateTimes:%s, DestDuty=%f, MeanDuty=%f' % (cnt, dest_duty, mean_duty)
            self.xls.setCell('Sheet1', self.row + cnt, 1, '%s' % ch_adj)
            self.xls.setCell('Sheet1', self.row + cnt, 2, '%s' % dest_duty)
            self.xls.setCell('Sheet1', self.row + cnt, 3, '%s' % cnt)
            self.xls.setCell('Sheet1', self.row + cnt, 4, '%s' % test_duty)
            self.xls.setCell('Sheet1', self.row + cnt, 5, '%s' % mean_duty)
            self.xls.setCell('Sheet1', self.row + cnt, 6, '%s ~ %s' % (min_duty, max_duty))
            if cnt >= 20:
                print 'Duty Calibrate %s%%: Fail\n' % dest_duty
                self.xls.setCell('Sheet1', self.row + 1, 7, 'Fail')
                self.fail_info.append(u'%s 方波占空比%s%%校正失败' % (ch_adj, dest_duty))
                self.fail_info.append('Square Wave Duty Calibrate: %s %s%% Fail' % (ch_adj, dest_duty))
                break
        if cnt < 20:
            print 'Duty Calibrate %s%%: Pass\n' % dest_duty
            self.xls.setCell('Sheet1', self.row + 1, 7, 'Pass')
        return test_duty

    def __duty_meas(self, ch_meas, duty_meas):
        #占空比测量
        self.SqrDuty.adjDuty(ch_meas, duty_meas, self.adj_sch)
        read_duty = self.SDS.get_value()
        if(abs(duty_meas-read_duty)>50):
            print('Send %s again!!!!!!' % duty_meas)
            self.SqrDuty.adjDuty(ch_meas, duty_meas, self.adj_sch)
            read_duty = self.SDS.get_value()
        
        duty = round(read_duty , 4)
        return duty

    def __SqrWaveDuty_test(self):
        #占空比检测
        self.sqrDuty_ch_list.reverse()
        cnt = 1
        for ch_meas in self.sqrDuty_ch_list:
            if ch_meas == self.sqrDuty_ch_list[1]:
                popup(u'注意! Notice!',u'请将示波器连接至%s 并断开另一通道\n\nPlease connect the DSO to %s and disconnect another channel' % (ch_meas, ch_meas))
            for test_duty in self.dutyCal_duty:
                self.SqrDuty.set_output_on(ch_meas)
                self.SqrDuty.set_duty(ch_meas, test_duty)
                time.sleep(2)
                meas_duty = self.SDS.get_value()
                max_testDuty = test_duty + 0.8
                min_testDuty = test_duty - 0.8
                self.xls.setCell('Sheet1', self.row + cnt, 1, '%s' % ch_meas)
                self.xls.setCell('Sheet1', self.row + cnt, 2, '%s' % test_duty)
                self.xls.setCell('Sheet1', self.row + cnt, 3, '%s' % meas_duty)
                self.xls.setCell('Sheet1', self.row + cnt, 4, '%s ~ %s' % (min_testDuty, max_testDuty))
                if meas_duty < min_testDuty or meas_duty > max_testDuty:
                    self.xls.setCell('Sheet1', self.row + cnt, 5, '%s' % 'Fail')
                    self.fail_info.append(u'%s 方波占空比%f%%检测失败' % (ch_meas, test_duty))
                    self.fail_info.append('Square Wave Duty Test: %s %f%% Fail' % (ch_meas, test_duty))
                    print ch_meas, test_duty, meas_duty, 'fail'
                else:
                    self.xls.setCell('Sheet1', self.row + cnt, 5, '%s' % 'Pass')
                    print ch_meas, test_duty, meas_duty, 'pass'
                cnt += 1
                self.SqrDuty.set_output_off(ch_meas)

    def saveAndClose(self):
        #保存报告文件并关闭
        self.xls.save()
        time.sleep(0.2)
        self.xls.close()
        time.sleep(0.2)

        self.SqrDuty.close()
        self.SDS.close()
        time.sleep(0.2)
        print 'The Square Wave Duty Calibrate Process is done'

    def show_result(self):
        if self.fail_info == []:
            testInfo_show([u'方波占空比校正通过，请进行下一步操作', 'Square Wave Duty Calibrate: Pass'])
        else:
            self.fail_info.insert(0, u'方波占空比校正失败\n')
            self.fail_info.insert(1, 'Square Wave Duty Calibrate: Fail\n')
            testInfo_show(self.fail_info)

if __name__ == '__main__':
    proc = DutyCalib_x()
    proc.DutyCal_main()
    proc.saveAndClose()
    proc.show_result()
