#!/usr/bin/env python
#-*- coding:utf-8 –*-
#-----------------------------------------------------------------------------
# $Header: $
#-----------------------------------------------------------------------------
# Summary:           Implementation of Feedback Channel Adjust
#
# Authors:           Jensen.Cai
#
# Started:           11/25/2014
#
# Version:           V1.7R1
#
# Copyright 2013-2016 Siglent Corporation. All Rights Reserved.
#
#-----------------------------------------------------------------------------

import sys, time
sys.path.append(sys.path[0]+'\\Prod Lib')
from reminder import popup
from test_info import testInfo_show
from win32com_excel import *
from device_remote import *
from param import *
from file import *

#-----------------------------------------------------------------------------

class FeedbackAdjust(FBParam):
    def __init__(self):
        super(FeedbackAdjust, self).__init__()
        self.FB = FBRemote()
        self.DMM = DMMRemote()
        self.fail_info = []
        self.row = 8

    def FBCal_main(self):
        self.DMM.set_default()
        self.DMM.set_mode('DC')
        
        #生成报告文件并将设备信息写入文件头
        report = TestReport('FeedbackAdjust')
        xls_name = report.create()
        self.xls = easyExcel(xls_name)
        report.head_write(self.xls)
        
        if self.adj_sch in ['1-0', '5-0', '5-1']:
            popup(u'注意! Notice!', u'请将万用表连接至通道一和通道二\n\nPlease connect the DMM to channel 1 and channel 2')
            if self.adj_sch == '5-0': 
                self.FB.set_output('C2', 'ON', 'HZ')
        else:
            popup(u'注意! Notice!', u'请将万用表连接至通道一，并保证通道二与万用表断开\n\nPlease connect the DMM to channel 1 and disconnect channel 2')
        
        
        print '\nStarting Feedback Adjust Process...\n'

        self.FB.init_step()
        self.row = 8
        for i in range(self.FB_step):
            self.__FB_adj(self.FB_volt[i])
            self.FB.adjust_step(i)
            self.row += 20
        self.FB.done_step()

        if self.adj_sch == '5-0': 
            self.FB.set_output_off('C2')

        self.FB.save_adjData()
        time.sleep(self.save_time)

    def __FB_adj(self, dest_volt):
        #反馈通道校正 采用逐步逼近法
        min_volt = dest_volt - self.FB_err
        max_volt = dest_volt + self.FB_err
        volt = dest_volt
        read_volt = self.__FB_meas(volt)
        cnt = 1
        print 'AdjustTimes:%s, SetVolt=%sV, ReadVolt=%.6fV' % (cnt, volt, read_volt)
        self.xls.setCell('Sheet1', self.row + cnt, 1, '%s' % dest_volt)
        self.xls.setCell('Sheet1', self.row + cnt, 2, '%s' % cnt)
        self.xls.setCell('Sheet1', self.row + cnt, 3, '%s' % volt)
        self.xls.setCell('Sheet1', self.row + cnt, 4, '%s' % read_volt)
        self.xls.setCell('Sheet1', self.row + cnt, 5, '%s ~ %s' % (min_volt, max_volt))
        while read_volt < min_volt or read_volt > max_volt :
            volt_step = (read_volt - dest_volt) / 2
            volt = volt - volt_step
            read_volt = self.__FB_meas(volt)
            cnt += 1
            print 'AdjustTimes:%s, SetVolt=%sV, ReadVolt=%.6fV' % (cnt, volt, read_volt)
            self.xls.setCell('Sheet1', self.row + cnt, 1, '%s' % dest_volt)
            self.xls.setCell('Sheet1', self.row + cnt, 2, '%s' % cnt)
            self.xls.setCell('Sheet1', self.row + cnt, 3, '%s' % volt)
            self.xls.setCell('Sheet1', self.row + cnt, 4, '%s' % read_volt)
            self.xls.setCell('Sheet1', self.row + cnt, 5, '%s ~ %s' % (min_volt, max_volt))
            if cnt >= 20:
                print 'Feedback Adjust %.1f V: Fail\n' % dest_volt
                self.xls.setCell('Sheet1', self.row + 1, 6, 'Fail')
                self.fail_info.append(u'反馈通道%sV校正失败' % dest_volt)
                self.fail_info.append('Feedback Channel Adjust: %sV Fail' % dest_volt)
                break
        if cnt < 20:
            print 'Feedback Adjust %.1f V: Pass\n' % dest_volt
            self.xls.setCell('Sheet1', self.row + 1, 6, 'Pass')

    def __FB_meas(self, set_val):
        #设置DC，万用表读值
        self.FB.set_adjDC(set_val)
        return self.DMM.get_value('DC')
    
    def saveAndClose(self):
        #保存报告文件并关闭
        self.xls.save()
        time.sleep(0.2)
        self.xls.close()
        time.sleep(0.2)

        self.FB.close()
        self.DMM.close()
        time.sleep(0.2)
        print 'The Feedback Adjust Process is done'

    def show_result(self):
        if self.fail_info == []:
            testInfo_show([u'反馈通道校正通过，请进行下一步操作', 'Feedback Channel Adjust: Pass'])
        else:
            self.fail_info.insert(0, u'反馈通道校正失败\n')
            self.fail_info.insert(1, 'Feedback Channel Adjust: Fail\n')
            testInfo_show(self.fail_info)

if __name__ == '__main__':
    proc = FeedbackAdjust()
    proc.FBCal_main()
    proc.saveAndClose()
    proc.show_result()
