#!/usr/bin/env python
#-*- coding:utf-8 –*-
#-----------------------------------------------------------------------------
# $Header: $
#-----------------------------------------------------------------------------
# Summary:              Implementation of Frequency Response Calibration
#
# Authors:              Jensen.Cai
#
# Started:              11/25/2014
#
# Version:              V1.7R1
#
# Copyright 2013-2016 Siglent Corporation. All Rights Reserved.
#
#-----------------------------------------------------------------------------

import sys, time, math
sys.path.append(sys.path[0]+'\\Prod Lib')
from reminder import popup
from test_info import testInfo_show
from win32com_excel import *
from device_remote import *
from param import *
from file import *

#-----------------------------------------------------------------------------
class FrqRespCalib(FRParam):
    def __init__(self):
        super(FrqRespCalib, self).__init__()
        self.FR = FRRemote()
        self.PowerSensor = PowerSensorRemote()
        self.min_ref = 0.0
        self.max_ref = 0.0
        self.ref_list = []
        self.row = 11
        self.fail_info = []

    def FRCal_main(self): 
        #生成报告文件并将设备信息写入文件头
        report = TestReport('FrqRespCalib')
        xls_name = report.create()
        self.xls = easyExcel(xls_name)
        report.head_write(self.xls)
        
        print '\nStarting Frequency Response Calibrate Process...'
        
        #开始频响校正，从第11行（Sheet1）开始写入数据
        self.row = 11
        self.PowerSensor.preset()

        self.__FR_Calib()
        
        self.FR.save_adjData()
        time.sleep(self.save_time)
        
        print '\nThe Frequency Response Calibrate Process is done'

        print '\nStarting Frequency Response Measure Process...'
        
        #开始频响检测，从第11行（Sheet2）开始写入数据
        self.row = 11
        self.PowerSensor.preset()
        self.__FR_Meas()

    def __FR_Calib(self):
        #频响校正 分大幅值和小幅值分别进行
        for ch_calib in self.FR_ch_list:
            self.FR.set_output_load(ch_calib, 50)
            self.FR.set_bswv(ch_calib, 'SINE', 1000000, 1)
            popup(u'注意! Notice!',u'请将功率计连接至%s\n\nPlease connect the power sensor to %s' % (ch_calib, ch_calib))
            self.FR.set_output_on(ch_calib)
            
            self.__PowerSensor_check()
            
            self.__FrqResp_calib(ch_calib)
    
            self.FR.set_output_off(ch_calib)

    def __PowerSensor_check(self):
        #检查功率计是否正常工作
        self.PowerSensor.set_frq(1000000)
        dBm_check = self.PowerSensor.get_value()
        if dBm_check < 3.0 or dBm_check > 5.0:
            popup(u'警告! Warning!', \
                  u'读值%s超出范围，请检查功率计是否正常工作！\n\n%s is out of range!Please check that the power sensor is operating well!' % (dBm_check, dBm_check))
            self.xls.save()
            self.xls.close()
            raw_input()
            exit()

    def __FrqResp_calib(self,  ch_adj):
        #先进行1M及以下频率点的校正，再按5M步进从5M频率点开始校正
        for state_adj in self.FR_state:
            maxFrq_adj = self.FR_max_frq[state_adj]
            amp_adj = self.FR_amp[state_adj]
            dBm_dest = self.FR_dbm[state_adj]
            if self.adj_sch == '2-0':
                step_calib = 10
                #SDG2000X新方案#
                if state_adj == 0:
                    self.PowerSensor.set_frq(self.ref_frq)
                    self.FR.set_frq(ch_adj, self.ref_frq)
                    self.FR.set_amp(ch_adj, amp_adj)
                    dBm_adj = self.PowerSensor.get_value()
                    dBm_dest = dBm_adj
            else:
                step_calib = 5

            for n in [1] + range(step_calib, maxFrq_adj + step_calib, step_calib):
                frq_adj = n * 1000000
                self.PowerSensor.set_frq(frq_adj)
                self.FR.set_flatness_bswv(ch_adj, frq_adj, amp_adj)
                print 'Calibrating Channel:%s, Vpp: %s V, Freq=%sMHz, DestAmp=%sdBm' % (ch_adj, amp_adj, n, dBm_dest)
                self.xls.setCell('Sheet1', self.row, 1, '%s' % ch_adj)
                self.xls.setCell('Sheet1', self.row, 2, '%s' % amp_adj)
                self.xls.setCell('Sheet1', self.row, 3, '%s' % frq_adj)
                idx_adj = n / step_calib
                comp_adj = self.__multiComp_calc(ch_adj, dBm_dest, amp_adj,  frq_adj)
                if self.adj_sch == '5-0':  
                    comp_adj = round(pow(10, comp_adj / 20), 4)
                self.FR.set_flatness_comp(ch_adj, state_adj, idx_adj, frq_adj, comp_adj)

    def __multiComp_calc(self,  ch_comp, dBm_comp, amp_comp,  frq_comp):
        #计算总补偿值 最多补偿15次，最后补偿值在[-0.05，0.05]范围内视为补偿完成
        comp = self.__comp_calc(dBm_comp)
        multi_comp = 0.0
        comp_min = -0.05
        comp_max = 0.05
        cnt = 0
        max_cnt = 15
        while comp < comp_min or comp > comp_max:
            multi_comp = multi_comp + comp
            if self.adj_sch == '5-0':  
                multi_comp = round(pow(10,  multi_comp / 20), 4)
            self.FR.set_multiComp(ch_comp, amp_comp, multi_comp)
            comp = self.__comp_calc(dBm_comp)
            cnt += 1
            if cnt == max_cnt: #SDG2000X新方案，若逼近次数达到15次，则加大误差范围至±0.2
                if self.adj_sch == '2-0' and frq_comp >= 100:
                    comp_min = -0.2
                    comp_max = 0.2
            if cnt > max_cnt:
                self.fail_info.append(u'%s 频响校正%s频率点失败' % (ch_comp, frq_comp))
                self.fail_info.append('Frequency Response Calibrate:%s %sHz Fail' % (ch_comp, frq_comp))
                break
        if comp > comp_min and comp < comp_max:
            multi_comp += comp
        print 'Total compensation is %sdB\n' % multi_comp
        return multi_comp

    def __comp_calc(self,  dBm_calc):
        #功率计读值，计算当前所需补偿值
        dBm_read = self.PowerSensor.get_value()
        comp_calc = dBm_calc - dBm_read
        print 'ReadAmp=%sdBm, compensation=%sdB' % (dBm_read, comp_calc)
        self.xls.setCell('Sheet1', self.row, 4, '%s' % dBm_read)
        self.xls.setCell('Sheet1', self.row, 5, '%s' % comp_calc)
        self.row += 1
        return comp_calc

    def __FR_Meas(self):
        #频响检测 分1M以下、1M以上和大幅值、小幅值分别进行
        self.FR_ch_list.reverse()
        for ch_meas in self.FR_ch_list:
            self.FR.set_output_load(ch_meas, 50)
            if ch_meas != self.FR_ch_list[0]:
                popup(u'注意! Notice!',u'请将功率计连接至%s\n\nPlease connect the power sensor to %s' % (ch_meas, ch_meas))
            self.FR.set_output_on(ch_meas)

            for j in range(0, len(self.FR_state)):
                print '\nChecking Channel:%s, Vpp: %s V' % (ch_meas, self.FR_amp[j])
                self.__ref_set(ch_meas, self.FR_state[j], self.FR_amp[j])
                self.__LF_meas(ch_meas, self.FR_state[j], self.FR_amp[j])
                self.__HF_meas(ch_meas, self.FR_state[j], self.FR_amp[j])

            if self.adj_sch in ['0-0', '1-0']:
                print '\nChecking Channel:%s, Vertical Accuracy' % ch_meas
                self.PowerSensor.set_frq(100000)
                self.FR.set_bswv(ch_meas, 'SINE', 100000, 1)
                self.__VertAcc_meas(ch_meas)
                print '\nThe Vertical Accuracy Measure Process is done'
    
            self.FR.set_output_off(ch_meas)

    def __ref_set(self,  ch_ref, state_ref, amp_ref):
        #设置平坦度参考值，以该点所测量的值为准判断是否通过
        print 'The Reference Frequency of Flatness Measure is %s' % self.ref_frq
        self.PowerSensor.set_frq(self.ref_frq)
        self.FR.set_bswv(ch_ref, 'SINE', self.ref_frq, amp_ref)
        dBm_get = self.PowerSensor.get_value()
        dBm_ref =  self.__ref_check(state_ref, dBm_get)
        self.ref_list.append(dBm_ref)
        print 'The Reference Value of Flatness Measure is %s' % dBm_ref
        self.xls.setCell('Sheet2', self.row, 1, '%s' % ch_ref)
        self.xls.setCell('Sheet2', self.row, 2, '%s' % amp_ref)
        self.xls.setCell('Sheet2', self.row, 3, '%s' % self.ref_frq)
        self.xls.setCell('Sheet2', self.row, 4, '%s' % dBm_ref)
        self.xls.setCell('Sheet2', self.row, 6, 'Reference')
        self.row += 1
        
    def __ref_check(self,  state_set,  dBm_set):
        if self.adj_sch == '2-0': #SDG2000X新方案#
            if state_set == 0:
                ref_err = 0.8
            elif state_set == 1:
                ref_err = 0.5
            elif state_set == 2:
                ref_err = 0.36
            else:
                ref_err = 0.24
        else:
            ref_err = 0.24
        ref_max = self.FR_dbm[state_set] + ref_err
        ref_min = self.FR_dbm[state_set] - ref_err
        if dBm_set < ref_max and dBm_set > ref_min:
            self.xls.setCell('Sheet2', self.row, 5, '%s ~ %s' % (ref_min, ref_max))
            return dBm_set
        else:
            popup(u'警告! Warning!', \
                  u'平坦度参考值%s超出范围!\n\nThe Reference Value of Flatness %s is out of range!' % (dBm_set, dBm_set))
            self.xls.save()
            self.xls.close()
            raw_input()
            exit()  

    def __LF_meas(self,  LF_ch, LF_state, LF_amp):
        #检测1M以下频响
        LF_stand = self.ref_list[-1]
        self.FR.set_amp(LF_ch, LF_amp)
        for low_frq in [50000, 100000]:
            self.PowerSensor.set_frq(low_frq)
            self.FR.set_frq(LF_ch, low_frq)
            LF_read = self.PowerSensor.get_value()
            LF_ShowFrq = low_frq / 1000
            if self.__dBm_check(LF_ShowFrq, LF_stand, LF_read, 'LF'):
                LF_result = 'Fail'
                self.fail_info.append(u'%s 频响%sKHz检测失败' % (LF_ch, LF_ShowFrq))
                self.fail_info.append('Flatness Measure: %s %sKHz Fail' % (LF_ch, LF_ShowFrq))
            else:
                LF_result = 'Pass'
            print 'Checking Channel:%s, TestAmp=%sV, TestFrq=%sKHz, ReadAmp=%sdBm, %s' % (LF_ch, LF_amp, LF_ShowFrq, LF_read, LF_result)
            self.xls.setCell('Sheet2', self.row, 1, '%s' % LF_ch)
            self.xls.setCell('Sheet2', self.row, 2, '%s' % LF_amp)
            self.xls.setCell('Sheet2', self.row, 3, '%s' % low_frq)
            self.xls.setCell('Sheet2', self.row, 4, '%s' % LF_read)
            self.xls.setCell('Sheet2', self.row, 6, '%s' % LF_result)
            self.row += 1

    def __HF_meas(self,  HF_ch, HF_state, HF_amp):
        HF_stand = self.ref_list[-1]
        MaxFrq_meas = self.FR_max_frq[HF_state]
        
        if self.adj_sch == '2-0' and HF_amp != 2.5: #SDG2000X在除2.5V外的其他档位只检测按10M步进校正的频率点
            step_meas = 10
            HF_range = [1] + range(step_meas, MaxFrq_meas + step_meas, step_meas)
        else:
            step_meas = 1
            HF_range = range(step_meas, MaxFrq_meas + step_meas, step_meas)
        
        self.FR.set_amp(HF_ch, HF_amp)
        for HF_ShowFrq in HF_range:
            high_frq = HF_ShowFrq * 1000000
            self.PowerSensor.set_frq(high_frq)
            self.FR.set_frq(HF_ch, high_frq)
            HF_read = self.PowerSensor.get_value()
            if self.__dBm_check(HF_ShowFrq, HF_stand, HF_read, 'HF'):
                HF_result = 'Fail'
                self.fail_info.append(u'%s 频响%sMHz检测失败' % (HF_ch, HF_ShowFrq))
                self.fail_info.append('Flatness Measure: %s %sMHz Fail' % (HF_ch, HF_ShowFrq))
            else:
                HF_result = 'Pass'
            print 'Checking Channel:%s, TestAmp=%sV, TestFrq=%sMHz, ReadAmp=%sdBm, %s' % (HF_ch, HF_amp, HF_ShowFrq, HF_read, HF_result)
            self.xls.setCell('Sheet2', self.row, 1, '%s' % HF_ch)
            self.xls.setCell('Sheet2', self.row, 2, '%s' % HF_amp)
            self.xls.setCell('Sheet2', self.row, 3, '%s' % high_frq)
            self.xls.setCell('Sheet2', self.row, 4, '%s' % HF_read)
            self.xls.setCell('Sheet2', self.row, 6, '%s' % HF_result)
            self.row += 1

    def __dBm_check(self,  frq, stand, dBm, flag):
        #判断检测是否通过
        if 'HF' == flag:
            if frq <= 160 and frq > 80:
                err_dB = self.FR_HF_err
            else:
                err_dB = self.FR_LF_err
            if self.adj_sch == '2-0': #SDG2000X大于100M平坦度较差
                if frq >= 100:
                    err_dB = 0.4
                else:
                    err_dB = self.FR_LF_err
        else:
            err_dB = self.FR_LF_err
        min_dBm = stand - err_dB
        max_dBm = stand + err_dB
        self.xls.setCell('Sheet2', self.row, 5, '%s ~ %s' % (min_dBm, max_dBm))
        if dBm < min_dBm or dBm > max_dBm:
            return 1
        else:
            return 0

    def __VertAcc_meas(self,  VA_ch):
        #功率计垂直精度测量
        if VA_ch == 'C1':
            VA_list = self.FR_ch1_VAlist
        elif VA_ch == 'C2':
            VA_list = self.FR_ch2_VAlist
        for VA_dBm in VA_list:
            self.FR.set_dbm(VA_ch, VA_dBm)
            VA_read = self.PowerSensor.get_value()
            if VA_dBm >= -4:
                VA_max = VA_dBm + 0.3
                VA_min = VA_dBm - 0.3
            elif VA_dBm == -12:
                VA_max = -11.65
                VA_min = -12.36
            elif VA_dBm == -22:
                VA_max = -21.53
                VA_min = -22.47
            elif VA_dBm == -28:
                VA_max = -27.36
                VA_min = -28.65
            elif VA_dBm == -36:
                VA_max = -34.87
                VA_min = -37.21
            if VA_read > VA_max or VA_read < VA_min:
                VA_result = 'Fail'
                self.fail_info.append(u'%s 垂直精度%sdBm档位检测失败' % (VA_ch, VA_dBm))
                self.fail_info.append('Vertical Accuracy Checking:%s %sdBm Fail' % (VA_ch, VA_dBm))
            else:
                VA_result = 'Pass'
            print 'Checking Channel:%s, SetAmp=%sdBm, ReadAmp=%sdBm, %s' % (VA_ch, VA_dBm, VA_read, VA_result)
            self.xls.setCell('Sheet2', self.row, 1, 'Vertical Accuracy')
            self.xls.setCell('Sheet2', self.row, 2, '%s' % VA_ch)
            self.xls.setCell('Sheet2', self.row, 3, '%s' % VA_dBm)
            self.xls.setCell('Sheet2', self.row, 4, '%s' % VA_read)
            self.xls.setCell('Sheet2', self.row, 5, '%s ~ %s' % (VA_min, VA_max))
            self.xls.setCell('Sheet2', self.row, 6, '%s' % VA_result)
            self.row += 1

    def saveAndClose(self):
        #保存报告文件并关闭
        self.xls.save()
        time.sleep(0.2)
        self.xls.close()
        time.sleep(0.2)
        
        self.FR.close()
        self.PowerSensor.close()
        time.sleep(0.2)
        
        print '\nThe Frequency Response Measure Process is done'

    def show_result(self):
        if self.fail_info == []:
            testInfo_show([u'频响校正和检测通过，请进行下一步操作', 'Frequency Response Calibrate: Pass'])
        else:
            self.fail_info.insert(0, u'频响校正和检测失败')
            self.fail_info.insert(1, 'Frequency Response Calibrate: Fail')
            testInfo_show(self.fail_info)

if __name__ == '__main__':
    proc = FrqRespCalib()
    proc.FRCal_main()
    proc.saveAndClose()
    proc.show_result()
