#!/usr/bin/env python
#-*- coding:utf-8 –*-
#-----------------------------------------------------------------------------
# $Header: $
#-----------------------------------------------------------------------------
# Summary:          Implementation of Report File Initialization
#
# Authors:          Jensen.Cai
#
# Started:          11/25/2014
#  
# Version:          V1.7R1
#
# Copyright 2013-2016 Siglent Corporation. All Rights Reserved.
#
#-----------------------------------------------------------------------------

import sys, os, shutil, time

from device_remote import *

#-----------------------------------------------------------------------------
class TestReport(SDGRemote):
    def __init__(self, project):
        super(TestReport, self).__init__()
        self.prj = project
        self.product = self.get_product()
        self.serial_num = self.get_SN()
        self.version = self.get_version()
        self.band = self.get_band()
    def create(self):
        #生成目录和报告文件
        os.system('taskkill /f /im excel.exe')       #关闭excel进程
        time.sleep(1)
        temp_dir = sys.path[0]+ '\\Excel Template'     #模板目录                  
        temp_file = temp_dir + '\\' + self.prj + '.xls'   #Excel模板
        init_dir = sys.path[0] + '..\\..\\Test Report' #原始目录，即报告的第一级目录
        test_dir = init_dir + '\\' + self.prj             #报告存储的第二级目录，以项目名称命名
        if os.path.exists(test_dir):
            pass
        else:
            os.makedirs(test_dir)
        file_dir = test_dir + '\\' + self.serial_num      #报告存储的第三级目录，以序列号命名
        if os.path.exists(file_dir):
            pass
        else:
            os.makedirs(file_dir)
        #拷贝模板文件为报告文件，命名方式为...\\序列号.xls ，第二次则为序列号_1.xls ，依此类推
        test_file = file_dir + '\\' + self.serial_num + '.xls'
        if not os.path.isfile(test_file):
            shutil.copy(temp_file, test_file)
        else:
            save_time = 1
            while save_time < 100:
                test_file = '%s\\%s_%s.xls' % (file_dir, self.serial_num, save_time)
                if not os.path.isfile(test_file):
                    break
                else:
                    save_time += 1
            shutil.copy(temp_file, test_file)
        return test_file
    
    def head_write(self, excel):
        #将主要信息写入文件头
        datetime = time.strftime('%Y-%m-%d %X', time.localtime())
        #根据项目名称写入对应设备信息
        if self.prj in ['FeedbackAdjust', 'DCMeasure', 'VertAccCalib', 'VertAccMeas']:
            dmm = DMMRemote()
            equip1 = dmm.get_idn()
        elif self.prj == 'FrqRespCalib':
            powersensor = PowerSensorRemote()
            equip1 = powersensor.get_idn()
            excel.setCell('Sheet1', 8, 2, '%sMHz' % self.band)
            excel.setCell('Sheet2', 4, 2, '%s' % self.product)
            excel.setCell('Sheet2', 4, 4, '%s' % self.serial_num)
            excel.setCell('Sheet2', 5, 2, '%s' % self.version)
            excel.setCell('Sheet2', 5, 4, '%s' % datetime)
            excel.setCell('Sheet2', 6, 2, '%s' % equip1)
            excel.setCell('Sheet2', 8, 2, '%sMHz' % self.band)
        elif self.prj == 'FrqRespMeas':
            powersensor = PowerSensorRemote()
            equip1 = powersensor.get_idn()
            excel.setCell('Sheet1', 8, 2, '%sMHz' % self.band)
        elif self.prj == 'DutyCalib':
            sds = SDSRemote()
            equip1 = sds.get_idn()
        elif self.prj == 'QA6':
            dmm = DMMRemote()
            equip1 = dmm.get_idn()
            sds = SDSRemote()
            equip2 = sds.get_idn()
            #equip3 = param.getInfo('SPD','info') # 此处由于SPD对ask操作的响应时间过长而不予适用
            excel.setCell('Sheet1', 7, 2, '%s' % equip2)
            #excel.setCell('Sheet1', 8, 2, '%s' % equip3)
        elif self.prj == 'FreqCounterCal':
            equip1 = self.get_idn()
        excel.setCell('Sheet1', 4, 2, '%s' % self.product)
        excel.setCell('Sheet1', 4, 4, '%s' % self.serial_num)
        excel.setCell('Sheet1', 5, 2, '%s' % self.version)
        excel.setCell('Sheet1', 5, 4, '%s' % datetime)
        excel.setCell('Sheet1', 6, 2, '%s' % equip1)
